<?php

namespace Drupal\ui_suite_uswds;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * UI Suite USWDS theme settings manager.
 */
class SettingsManager implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * Constructs a SettingsManager object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(protected ModuleHandlerInterface $moduleHandler, protected EntityTypeManagerInterface $entityTypeManager) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    // @phpstan-ignore-next-line
    return new static(
      $container->get('module_handler'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * Return menu Settings.
   */
  protected function menuSettingsForm($title, $value): array {
    try {
      $all_menus = $this->entityTypeManager->getStorage('menu')->loadMultiple();
      $menus = [
        "" => "(None)",
      ];
      foreach ($all_menus as $id => $menu) {
        $menus[$id] = $menu->label();
      }
      asort($menus);
      return [
        '#type' => 'select',
        '#title' => $title,
        '#options' => $menus,
        '#default_value' => $value,
      ];
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException) {
    }
    return [];
  }

  /**
   * Alters theme settings form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param string $form_id
   *   The form id.
   *
   * @see hook_form_alter()
   */
  public function themeSettingsAlter(array &$form, FormStateInterface $form_state, $form_id): void {
    // CDN Provider.
    $form['cdn_settings'] = [
      '#type' => 'details',
      '#title' => t('CDN Settings'),
      '#open' => FALSE,
      'menu-help' => [
        '#markup' => t('NOTE: The "CDN Provider" options below is an option for loading USWDS CSS and JS. Using a "CDN Provider" for loading USWDS externally, like jsDelivr, should not be considered a good option for a production website. Please, follow the instructions and download your USWDS files.'),
      ],
      'cdn_provider' => [
        '#type' => 'select',
        '#title' => t('CDN Provider'),
        '#description' => t('Choose between a custom CDN source or none. None will expect USWDS through /package/dist folder or sass.'),
        '#options' => [
          'none' => t('- None -'),
          'custom' => t('Custom'),
        ],
        '#default_value' => theme_get_setting('cdn_provider'),
      ],
      'cdn_custom_img' => [
        '#type' => 'textfield',
        '#title' => t('USWDS Image URL'),
        '#default_value' => theme_get_setting('cdn_custom_img'),
        '#states' => [
          'visible' => [
            ':input[name="cdn_provider"]' => ['value' => 'custom'],
          ],
        ],
      ],
      'cdn_custom_css' => [
        '#type' => 'textfield',
        '#title' => t('USWDS CSS URL'),
        '#description' => t('It is best to use https protocols here as it will allow more flexibility if the need ever arises.'),
        '#default_value' => theme_get_setting('cdn_custom_css'),
        '#states' => [
          'visible' => [
            ':input[name="cdn_provider"]' => ['value' => 'custom'],
          ],
        ],
      ],
      'cdn_custom_js' => [
        '#type' => 'textfield',
        '#title' => t('USWDS JavaScript URL'),
        '#description' => t('It is best to use https protocols here as it will allow more flexibility if the need ever arises.'),
        '#default_value' => theme_get_setting('cdn_custom_js'),
        '#states' => [
          'visible' => [
            ':input[name="cdn_provider"]' => ['value' => 'custom'],
          ],
        ],
      ],
    ];

    // Header style.
    $form['header_style_fieldset'] = [
      '#type' => 'details',
      '#title' => t('Header settings'),
      '#open' => FALSE,
      'uswds_header_style' => [
        '#type' => 'select',
        '#required' => TRUE,
        '#title' => t('Choose a style of header to use'),
        '#options' => [
          'basic' => t('Basic'),
          'extended' => t('Extended (Default)'),
        ],
        '#default_value' => theme_get_setting('uswds_header_style') ?? 'extended',
      ],
      'uswds_primary_menu' => $this->menuSettingsForm(
        $this->t("Primary menu"),
        theme_get_setting('uswds_primary_menu') ?? "main"
      ),
      'uswds_secondary_menu' => $this->menuSettingsForm(
        $this->t("Secondary menu"),
        theme_get_setting('uswds_secondary_menu') ?? "account"
      ),
      'uswds_header_mega' => [
        '#type' => 'checkbox',
        '#title' => t('Use megamenu in the header?'),
        '#description' => t('Site building note: Megamenu require hierarchical three-level menus, where the second level of menu items are used to determine the "columns" for the megamenu.'),
        '#default_value' => theme_get_setting('uswds_header_mega'),
      ],
      'uswds_government_banner' => [
        '#type' => 'checkbox',
        '#title' => t('Display the official U.S. government banner at the top of each page.'),
        '#default_value' => theme_get_setting('uswds_government_banner'),
      ],
    ];

    if (!$this->moduleHandler->moduleExists('search')) {
      $form['header_style_fieldset']['uswds_search']['#description'] = t('Requires the core Search module to be enabled.');
    }

    // Footer style.
    $form['footer_style_fieldset'] = [
      '#type' => 'details',
      '#title' => t('Footer settings'),
      '#open' => FALSE,
      'uswds_footer_style' => [
        '#type' => 'select',
        '#required' => TRUE,
        '#title' => t('Choose a style of footer to use'),
        '#description' => t('See <a href="https://designsystem.digital.gov/components/footer/">USWDS Footer </a> for more details'),
        '#options' => [
          'big' => t('Big (default)'),
          'medium' => t('Medium'),
          'slim' => t('Slim'),
        ],
        '#default_value' => theme_get_setting('uswds_footer_style') ?? 'big',
      ],
      'uswds_footer_style_help' => [
        '#type' => 'markup',
        '#markup' => '<span>Using "Big" option will make first menu items appear as headers</span><br><span>Using "Medium" or "Slim" options will only display first level of menu.</span>',
      ],
      'uswds_footer_menu' => $this->menuSettingsForm(
        $this->t("Footer menu"),
        theme_get_setting('uswds_footer_menu') ?? "main"
      ),
      'uswds_footer_agency_name' => [
        '#type' => 'textfield',
        '#title' => t('Footer agency name'),
        '#default_value' => theme_get_setting('uswds_footer_agency_name'),
        '#states' => [
          'visible' => [
            'select[name="uswds_footer_style"]' => ['value' => 'big'],
          ],
        ],
      ],
      'uswds_footer_agency_url' => [
        '#type' => 'textfield',
        '#title' => t('Footer agency URL'),
        '#default_value' => theme_get_setting('uswds_footer_agency_url'),
      ],
      'uswds_footer_agency_logo' => [
        '#type' => 'textfield',
        '#title' => t("Path to footer agency logo (from this theme's folder)"),
        '#description' => t('For example: images/footer-agency.png'),
        '#default_value' => theme_get_setting('uswds_footer_agency_logo'),
      ],
      'uswds_contact_center' => [
        '#type' => 'textfield',
        '#title' => t('Name of contact center'),
        '#default_value' => theme_get_setting('uswds_contact_center'),
      ],
      'uswds_email' => [
        '#type' => 'textfield',
        '#title' => t('Email'),
        '#default_value' => theme_get_setting('uswds_email'),
        '#states' => [
          'visible' => [
            'select[name="uswds_footer_style"]' => ['value' => 'big'],
          ],
        ],
      ],
      'uswds_phone' => [
        '#type' => 'textfield',
        '#title' => t('Phone'),
        '#default_value' => theme_get_setting('uswds_phone'),
        '#states' => [
          'visible' => [
            'select[name="uswds_footer_style"]' => ['value' => 'big'],
          ],
        ],
      ],
      'uswds_social_links_group' => [
        '#type' => 'fieldset',
        '#title' => t('Social media links'),
        '#states' => [
          'visible' => [
            'select[name="uswds_footer_style"]' => ['value' => 'big'],
          ],
        ],
        'uswds_facebook' => [
          '#type' => 'textfield',
          '#title' => t('Facebook link'),
          '#default_value' => theme_get_setting('uswds_facebook'),
          '#states' => [
            'visible' => [
              'select[name="uswds_footer_style"]' => ['value' => 'big'],
            ],
          ],
        ],
        'uswds_instagram' => [
          '#type' => 'textfield',
          '#title' => t('Instagram'),
          '#default_value' => theme_get_setting('uswds_instagram'),
          '#states' => [
            'visible' => [
              'select[name="uswds_footer_style"]' => ['value' => 'big'],
            ],
          ],
        ],
        'uswds_linkedin' => [
          '#type' => 'textfield',
          '#title' => t('Linkedin'),
          '#default_value' => theme_get_setting('uswds_linkedin'),
          '#states' => [
            'visible' => [
              'select[name="uswds_footer_style"]' => ['value' => 'big'],
            ],
          ],
        ],
        'uswds_twitter' => [
          '#type' => 'textfield',
          '#title' => t('Twitter link'),
          '#default_value' => theme_get_setting('uswds_twitter'),
          '#states' => [
            'visible' => [
              'select[name="uswds_footer_style"]' => ['value' => 'big'],
            ],
          ],
        ],
        'uswds_youtube' => [
          '#type' => 'textfield',
          '#title' => t('Youtube link'),
          '#default_value' => theme_get_setting('uswds_youtube'),
          '#states' => [
            'visible' => [
              'select[name="uswds_footer_style"]' => ['value' => 'big'],
            ],
          ],
        ],
        'uswds_rss' => [
          '#type' => 'textfield',
          '#title' => t('RSS feed'),
          '#default_value' => theme_get_setting('uswds_rss'),
          '#states' => [
            'visible' => [
              'select[name="uswds_footer_style"]' => ['value' => 'big'],
            ],
          ],
        ],
      ],
    ];

    // Menu settings.
    $saved_bypass = theme_get_setting('uswds_menu_bypass');
    if (empty($saved_bypass)) {
      $saved_bypass = [];
    }

    $form['menu_fieldset'] = [
      '#type' => 'details',
      '#title' => t('Menu settings'),
      '#open' => FALSE,
      'uswds_menu_bypass' => [
        '#type' => 'checkboxes',
        '#title' => t('Bypass USWDS menu processing for these menus.'),
        '#description' => t('Choose the menus which you would like to exempt from USWDS alterations. Note that a cache clear may be necessary after changing these settings.'),
        '#options' => [
          'primary_menu' => t('Primary menu'),
          'secondary_menu' => t('Secondary menu'),
          'sidebar_first' => t('Sidebar First menu'),
          'sidebar_second' => t('Sidebar Second menu'),
          'footer_primary' => t('Footer primary'),
        ],
        '#default_value' => $saved_bypass,
      ],
    ];

    $form['subtheme'] = [
      '#type' => 'details',
      '#title' => $this->t('Subtheme'),
      '#description' => $this->t("Create subtheme."),
      '#open' => FALSE,
    ];

    $form['subtheme']['subtheme_folder'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Subtheme location'),
      '#default_value' => 'themes/custom',
      '#description' => $this->t("Relative path to the webroot <em>%root</em>. No trailing slash.", [
        '%root' => DRUPAL_ROOT,
      ]),
    ];

    $form['subtheme']['subtheme_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Subtheme name'),
      '#default_value' => 'UI Suite USWDS subtheme',
      '#description' => $this->t("If name is empty, machine name will be used."),
    ];

    $form['subtheme']['subtheme_machine_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Subtheme machine name'),
      '#default_value' => 'ui_suite_uswds_subtheme',
      '#description' => $this->t("Use only lower-case letters, numbers, and underscores. Name must start with a letter. For details, see <a href='https://www.drupal.org/docs/creating-custom-modules/naming-and-placing-your-drupal-module'>Naming and placing your Drupal module</a>."),
    ];

    $form['subtheme']['create'] = [
      '#type' => 'submit',
      '#name' => 'subtheme_create',
      '#value' => $this->t('Create'),
      '#button_type' => 'danger',
      '#attributes' => [
        'class' => ['btn btn-danger'],
      ],
      '#submit' => ['ui_suite_uswds_form_system_theme_settings_subtheme_submit'],
      '#validate' => ['ui_suite_uswds_form_system_theme_settings_subtheme_validate'],
    ];
  }

}
